//
//  MY_Knob.m
//
//  Modified code based on auguiCocoa: Created by Airy Andr on Fri May 21 2004
//  Modified by Alex Kenis 2014
//
//  Code has been modified for ObjC ARC.  Uncomment management features for non-ARC use

#import "MY_Knob.h"

@implementation MY_KnobCell

// ---------------------------------------------------------
//  Initialization
// ---------------------------------------------------------
- (void) normalizeValue{
    if (_minValue >= _maxValue)
        _maxValue = _minValue + 0.00001;
    if (_minValue > _value)
        _value = _minValue;
    if (_maxValue < _value)
        _value = _maxValue;
}

// SET DEFAULT VALUES HERE
- (void) defaultValues
{
    [self setKnobImage:0];
    [self setBackgroundImage:0];
    
    [self setKnobName:@"defaultKnob"];
    [self setBackgroundName:@"defaultKnobBack"];
    
    [self setMinAngle:-135.];
    [self setMaxAngle:135.];
    
    [self setXOffset:0.];
    [self setYOffset:0.];
    
    [self setDoubleValue:0.];
    
    [self setIntegralValues:FALSE];
}

- (id)init {
    self = [super init];
    if (self)
    {
        [self defaultValues];
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)coder {
    self = [super initWithCoder:coder];
    [self defaultValues];
    [self normalizeValue];
    return self;
}

- (id)copyWithZone:(NSZone *)zone {
    MY_KnobCell *newCopy = [[MY_KnobCell alloc] init];
    [newCopy setBackgroundName:_backgroundName];
    [newCopy setKnobName:_knobName];
    return newCopy;
}

- (void)dealloc {
    /*[super dealloc];*/ // for non-arc
}

- (void)sendActionToTarget {
    if ([self target] && [self action]) {
        [(NSControl *)[self controlView] sendAction:[self action] to:[self target]];
    }
}

- (void)setBackgroundName:(NSString *)name {
    /*[name retain];*/ // for non-arc
    /*[_backgroundName release];*/ // for non-arc
    _backgroundName = name;
    NSImage *image = [NSImage imageNamed:name];
    if (!image && [name isEqualToString:@"defaultKnobBack"]) {
        NSBundle* bundle = [NSBundle bundleForClass: [self class]];
        image = /*[*/[[NSImage alloc] initByReferencingFile: [bundle pathForImageResource: name]]/* autorelease]*/; // for non-arc
    }
    [self setBackgroundImage:image];
}

- (void)setKnobName:(NSString *)name {
    /*[name retain];*/ // for non-arc
    /*[_knobName release];*/ // for non-arc
    _knobName = name;
    NSImage *image = [NSImage imageNamed:name];
    if (!image && [name isEqualToString:@"defaultKnob"]) {
        NSBundle* bundle = [NSBundle bundleForClass: [self class]];
        image = /*[*/[[NSImage alloc] initByReferencingFile: [bundle pathForImageResource: name]] /*autorelease]*/; // for non-arc
    }
    [self setKnobImage:image];
}

- (void)setBackgroundImage:(NSImage *)image {
    /*[image retain];*/ // for non-arc
    /*[_background release];*/ // for non-arc
    _background = image;
}

- (void)setKnobImage:(NSImage *)image {
    /*[image retain];*/ // for non-arc
    /*[_knob release];*/ // for non-arc
    _knob = image;
}

- (NSString *)backgroundName {
    return /*[[*/_backgroundName/* retain] autorelease]*/; // for non-arc
}

- (NSString *)knobName {
    return /*[[*/_knobName/* retain] autorelease]*/; // for non-arc
}

- (NSImage *)background {
    return /*[[*/_background/* retain] autorelease]*/; // for non-arc
}

- (NSImage *)knob {
    return /*[[*/_knob/* retain] autorelease]*/; // for non-arc
}

- (double) doubleValue{
    return _value;
}

- (double)minValue{
    return _minValue;
}

- (double)maxValue{
    return _maxValue;
}

- (void)setDoubleValue:(double)val{
    _value = val;
    [self normalizeValue];
    [self sendActionToTarget];
}

- (void)setMinValue:(double)min{
    _minValue = min;
 [self normalizeValue];
}

- (void)setMaxValue:(double)max{
    _maxValue = max;
    [self normalizeValue];
}

- (double)minAngle{
    return _minAngle;
}

- (double)maxAngle{
    return _maxAngle;
}

- (void)setMinAngle:(double)min{
    _minAngle = min;
}

- (void)setMaxAngle:(double)max{
    _maxAngle = max;
}

- (double)xOffset{
    return _xOffset;
}

- (double)yOffset{
    return _yOffset;
}

- (void)setXOffset:(double)offset{
    _xOffset = offset;
}

- (void)setYOffset:(double)offset{
    _yOffset = offset;
}

- (BOOL)integralValues{
    return _integralValues;
}

- (void)setIntegralValues:(BOOL)integral{
    _integralValues = integral;
}

- (BOOL)startTrackingAt:(NSPoint)startPoint inView:(NSView *)controlView
{
    return TRUE;
}

- (BOOL)continueTracking:(NSPoint)lastPoint at:(NSPoint)currentPoint inView:(NSView *)controlView
{
    double deltaX = currentPoint.x - lastPoint.x;
    double deltaY = currentPoint.y - lastPoint.y;
     deltaX = -deltaX; // delete these for inverted Y axis mouse tracking
     deltaY = -deltaY; // delete these for inverted Y axis mouse tracking
    double value = [self doubleValue];
    value += (deltaX + deltaY) * ([self maxValue]-[self minValue])/300.;
    [self setDoubleValue:value];
    [ controlView setNeedsDisplay:TRUE ];
    return true;
}

- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView untilMouseUp:(BOOL)flag
{
    // We want to continue tracking event when the mouse is outside the view
    return [super trackMouse:theEvent inRect:NSMakeRect(-1e10,-1e10,2e10,2e10) ofView:controlView untilMouseUp:YES];
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
    {
    if (_background) {
        [_background drawInRect:cellFrame fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0 respectFlipped:YES hints:NULL];
        
    }
    // Drawing knob
    if (_knob) {
        // Rotating context
        NSAffineTransform* trans = [NSAffineTransform transform];
        [trans translateXBy:[self xOffset] + cellFrame.size.width * 0.5 yBy:[self yOffset]+cellFrame.size.height * 0.5];
        double theRotation = [self minAngle]+(([self doubleValue]-[self minValue])/([self maxValue]-[self minValue]))*([self maxAngle]-[self minAngle]);
        
        [trans rotateByDegrees:theRotation]; // ACK note: change to -theRotation to reverse the max-min rotation direction
        [trans translateXBy:-cellFrame.size.width * 0.5 yBy:-cellFrame.size.height * 0.5];
        [trans translateXBy:-1 yBy:1];
        [trans concat];
        [_knob drawInRect:cellFrame fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0 respectFlipped:YES hints:NULL]; // ACK note: respectFlipped:YES hints:NULL turns cell upright
    }
}

@end

@implementation MY_Knob

+ (void)initialize {
    if (self == [MY_Knob class]) {
        [self setCellClass: [MY_KnobCell class]];
    }
}

+ (Class)cellClass {
    return [MY_KnobCell class];
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
        /*[super dealloc];*/ // for non-arc
}

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
        
    }
    return self;
}

- (void)setBackgroundName:(NSString *)name {
    [[self cell] setBackgroundName:name];
    [self setNeedsDisplay:TRUE];
}

- (void)setKnobName:(NSString *)name {
    [[self cell] setKnobName:name];
    [self setNeedsDisplay:TRUE];
}

- (void)setBackgroundImage:(NSImage *)image {
    [[self cell] setBackgroundImage:image];
    [self setNeedsDisplay:TRUE];
}

- (void)setKnobImage:(NSImage *)image {
    [[self cell] setKnobImage:image];
    [self setNeedsDisplay:TRUE];
}

- (NSString *)backgroundName {
    return [[self cell] backgroundName];
}

- (NSString *)knobName {
    return [[self cell] knobName];
}

- (NSImage *)background {
    return [[self cell] background];
}

- (NSImage *)knob {
    return [[self cell] knob];
}

- (double)doubleValue{
    return [[self cell] doubleValue];
}

- (float)floatValue{
    return [[self cell] doubleValue];
}

- (double)minValue{
    return [[self cell] minValue];
}

- (double)maxValue{
    return [[self cell] maxValue];
}

- (void)setMinValue:(double)min{
    [[self cell] setMinValue:min];
    [self setNeedsDisplay:TRUE];
}

- (void)setDoubleValue:(double)val{
    [[self cell] setDoubleValue:val];
    [self setNeedsDisplay:TRUE];
}

- (void)setFloatValue:(float)val{
    [[self cell] setDoubleValue:val];
    [self setNeedsDisplay:TRUE];
}


- (void)setMaxValue:(double)max{
    [[self cell] setMaxValue:max];
    [self setNeedsDisplay:TRUE];
}

- (double)minAngle{
    return [[self cell] minAngle];
}

- (double)maxAngle{
    return [[self cell] maxAngle];
}

- (void)setMinAngle:(double)min{
    [[self cell] setMinAngle:min];
    [self setNeedsDisplay:TRUE];
}

- (void)setMaxAngle:(double)max{
    [[self cell] setMaxAngle:max];
    [self setNeedsDisplay:TRUE];
}

- (double)xOffset{
    return [[self cell] xOffset];
}

- (double)yOffset{
    return [[self cell] yOffset];
}

- (void)setYOffset:(double)offset{
    [[self cell] setYOffset:offset];
    [self setNeedsDisplay:TRUE];
}

- (void)setXOffset:(double)offset{
    [[self cell] setXOffset:offset];
    [self setNeedsDisplay:TRUE];
}

- (BOOL)integralValues{
    return [[self cell] integralValues];
}

- (void)setIntegralValues:(BOOL)integral{
    [[self cell] setIntegralValues:integral];
    [self setNeedsDisplay:TRUE];
}

@end
